@@ -16,12 +16,17 @@ enum EXTTIME_MODE {
   EXTTIME_NONE,EXTTIME_1S,EXTTIME_HIGH1,EXTTIME_HIGH2,EXTTIME_HIGH3
 };
 enum {NAMES_ORIGINALCASE,NAMES_UPPERCASE,NAMES_LOWERCASE};
-enum MESSAGE_TYPE {MSG_STDOUT,MSG_STDERR,MSG_NULL};
-enum OVERWRITE_MODE { OVERWRITE_ASK,OVERWRITE_ALL,OVERWRITE_NONE};
+enum MESSAGE_TYPE {MSG_STDOUT,MSG_STDERR,MSG_ERRONLY,MSG_NULL};
+enum OVERWRITE_MODE {
+  OVERWRITE_ASK,OVERWRITE_ALL,OVERWRITE_NONE,OVERWRITE_AUTORENAME
+};
+
+enum RAR_CHARSET { RCH_DEFAULT=0,RCH_ANSI,RCH_OEM,RCH_UNICODE };
 
 #define     MAX_FILTERS           16
 enum FilterState {FILTER_DEFAULT=0,FILTER_AUTO,FILTER_FORCE,FILTER_DISABLE};
 
+
 struct FilterMode
 {
   FilterState State;
@@ -38,12 +43,16 @@ class RAROptions
     void Init();
 
     uint ExclFileAttr;
+    uint InclFileAttr;
+    bool InclAttrSet;
     uint WinSize;
     char TempPath[NM];
     char SFXModule[NM];
     char ExtrPath[NM];
     wchar ExtrPathW[NM];
     char CommentFile[NM];
+    RAR_CHARSET CommentCharset;
+    RAR_CHARSET FilelistCharset;
     char ArcPath[NM];
     char Password[MAXPASSWORD];
     bool EncryptHeaders;
@@ -55,6 +64,8 @@ class RAROptions
     int Recovery;
     int RecVolNumber;
     bool DisablePercentage;
+    bool DisableCopyright;
+    bool DisableDone;
     int Solid;
     int SolidCount;
     bool ClearArc;
@@ -92,6 +103,8 @@ class RAROptions
     uint FileTimeNewer;
     RarTime FileTimeBefore;
     RarTime FileTimeAfter;
+    Int64 FileSizeLess;
+    Int64 FileSizeMore;
     bool OldNumbering;
     bool Lock;
     bool Test;
@@ -108,6 +121,11 @@ class RAROptions
     EXTTIME_MODE xarctime;
     char CompressStdin[NM];
 
+#ifdef PACK_SMP
+    uint Threads;
+#endif
+
+
 
 
 #ifdef RARDLL
