@@ -1,8 +1,7 @@
 #include "rar.hpp"
 
-ComprDataIO::ComprDataIO(CmdAdd *Command)
+ComprDataIO::ComprDataIO()
 {
-  ComprDataIO::Command=Command;
   Init();
 }
 
@@ -21,7 +20,7 @@ void ComprDataIO::Init()
   SrcFile=NULL;
   DestFile=NULL;
   UnpWrSize=0;
-  ComprDataIO::Command=Command;
+  Command=NULL;
   Encryption=0;
   Decryption=0;
   TotalPackRead=0;
@@ -119,9 +118,13 @@ void ComprDataIO::UnpWrite(byte *Addr,uint Count)
       ErrHandler.Exit(USER_BREAK);
     if (Cmd->ProcessDataProc!=NULL)
     {
+#if defined(_WIN_32) && !defined(_MSC_VER) && !defined(__MINGW32__)
       _EBX=_ESP;
+#endif
       int RetCode=Cmd->ProcessDataProc(Addr,Count);
+#if defined(_WIN_32) && !defined(_MSC_VER) && !defined(__MINGW32__)
       _ESP=_EBX;
+#endif
       if (RetCode==0)
         ErrHandler.Exit(USER_BREAK);
     }
@@ -208,20 +211,20 @@ void ComprDataIO::GetUnpackedData(byte **Data,uint *Size)
 }
 
 
-void ComprDataIO::SetEncryption(int Method,char *Password,byte *Salt,bool Encrypt)
+void ComprDataIO::SetEncryption(int Method,char *Password,byte *Salt,bool Encrypt,bool HandsOffHash)
 {
   if (Encrypt)
   {
     Encryption=*Password ? Method:0;
 #ifndef NOCRYPT
-    Crypt.SetCryptKeys(Password,Salt,Encrypt);
+    Crypt.SetCryptKeys(Password,Salt,Encrypt,false,HandsOffHash);
 #endif
   }
   else
   {
     Decryption=*Password ? Method:0;
 #ifndef NOCRYPT
-    Decrypt.SetCryptKeys(Password,Salt,Encrypt,Method<29);
+    Decrypt.SetCryptKeys(Password,Salt,Encrypt,Method<29,HandsOffHash);
 #endif
   }
 }
@@ -254,3 +257,4 @@ void ComprDataIO::SetUnpackToMemory(byte *Addr,uint Size)
   UnpackToMemorySize=Size;
 }
 
+
